//
//  NSArray-NisusAdditions.h
//  WriterMerged
//
//  Created by Martin Wierschin on 2006.03.01.
//  Copyright 2006 Nisus Software, Inc. All rights reserved.
//

#import <Cocoa/Cocoa.h>

typedef NSComparisonResult (*NTCompareFunction)(id obj1, id obj2, void* context);
NSComparisonResult NTCompareUsingSelector( id obj1, id obj2, void* context );




@interface NSArray (NisusCore)

+ (NSArray*) arrayByAddingObjectNisus:(id)obj toArray:(NSArray*)array ;
- (NSArray*) arrayByRemovingObjectNisus:(id)obj ;
- (NSArray*) arrayByMovingObjects:(NSArray*)moveObjects toDestinationIndexNisus:(NSUInteger)insertIndex ;
- (NSArray*) subarrayWithObjectsPassingTestNisus:(BOOL(^)(id obj, NSUInteger objIndex, BOOL* stop))critiera ;

- (NSRange) rangeAtIndex:(NSUInteger)objIndex ;
- (BOOL) enumerateRangesNisusUsing:(void(^)(NSRange aRange, NSUInteger rangeIndex, BOOL* stop))visitor ;

- (NSUInteger) nearestIndexOfSortedObjectNisus:(id)obj withComparator:(NSComparator)comp ;
- (NSUInteger) nearestIndexOfSortedObjectNisus:(id)obj withCompareSelector:(SEL)sel ;
- (NSUInteger) indexOfSortedObjectNisus:(id)obj withComparator:(NSComparator)comp ;
- (NSUInteger) indexOfSortedObjectNisus:(id)obj withCompareSelector:(SEL)sel ;

- (BOOL) containsObjectIdenticalToNisus:(id)obj ;
- (NSUInteger) indexOfObjectNisus:(id)obj withCompareSelector:(SEL)sel ;

- (void) enumerateObjectsInReverseNisusUsing:(void(^)(id obj, NSUInteger objIndex, BOOL* stop))visitor ;
- (void) enumerateObjectsInReverseNisus:(BOOL)isReverse using:(void(^)(id obj, NSUInteger objIndex, BOOL* stop))visitor ;

@end



@interface NSMutableArray (NisusAdditions)
- (id) initWithSingleObject:(id)obj count:(NSUInteger)count ;

- (void) addObjectIfNotNilNisus:(id)obj ;
- (void) moveObjectAtIndex:(NSUInteger)curIdx toDestinationIndex:(NSUInteger)destIdx ;

// sorted arrays
- (NSUInteger) insertSortedObjectNisus:(id)obj withComparator:(NSComparator)comp ;
- (BOOL) removeSortedObjectNisus:(id)obj withComparator:(NSComparator)comp ;

- (NSUInteger) insertSortedObjectNisus:(id)obj withCompareSelector:(SEL)sel ;
- (BOOL) removeSortedObjectNisus:(id)obj withCompareSelector:(SEL)sel ;

// reordering
- (void) randomizeNisus ;
- (void) reverseNisus ;
@end



// These work for any NSArray/NSSet/etc (eg: any object that implements -objectEnumerator)
@interface NSObject (NisusCollectionAdditions)
- (NSArray*) arrayByMakingObjectsPerformSelector:(SEL)sel ;
- (NSArray*) arrayByMakingObjectsPerformSelector:(SEL)sel withObject:(id)obj ;
- (NSArray*) arrayByFilteringObjectsNisus:(id(^)(id object))filter ;

- (NSSet*) setByMakingObjectsPerformSelector:(SEL)sel ;
- (NSSet*) setByMakingObjectsPerformSelector:(SEL)sel withObject:(id)obj ;
- (NSSet*) setByFilteringObjectsNisus:(id(^)(id object))filter ;
@end


// Convenience methods for creating sort descriptors
@interface NSSortDescriptor (NisusAdditions)
+ (NSSortDescriptor*) sortDescriptorWithKey:(NSString*)key selector:(SEL)sel ;

+ (NSSortDescriptor*) sortDescriptorWithNaturalSortStringKey:(NSString*)key ascending:(BOOL)isAsc ;
+ (NSSortDescriptor*) sortDescriptorWithNaturalSortStringKey:(NSString*)key ;
@end

