//
//  NSArray+NisusRanges.h
//  WriterMerged
//
//  Created by Martin Wierschin on 2005.12.02.
//  Copyright 2005 Nisus Software, Inc. All rights reserved.

// This category makes it easier to work with arrays that hold boxed NSRange values (eg: as NSValue)
// The concrete NSArray class NisusRangeArray overrides these methods to be more efficient.

#import <Cocoa/Cocoa.h>



/***********************************
 * Range Array Primitives
 ***********************************/

@interface NSArray (NisusRangeArrayPrimitives)

+ (NSArray*) arrayWithZeroRange ;
+ (NSArray*) arrayWithRange:(NSRange)rng ;

- (NSRange) rangeAtIndex:(NSUInteger)idx ;
- (void) getRanges:(NSRange*)buffer inRange:(NSRange)rng ;
- (void) accessRangeBufferUsing:(void(^)(const NSRange* buffer, NSUInteger bufferCount))visitor ;

@end



@interface NSArray (NisusRangeArray)

- (NSArray*) arrayByNormalizingRanges ;
- (NSArray*) arrayByInvertingRangesWithinLimit:(NSUInteger)lim ;

- (BOOL) isNormalizedRangeArray ;

// these methods assume the range is sorted
- (NSUInteger) insertionIndexForSortedRange:(NSRange)rng ;
- (NSUInteger) insertionIndexForBoundSortedRange:(NSRange)rng ;
- (NSRange) indexesOfSortedRangesIntersectingRange:(NSRange)testRng ;
- (NSRange) indexesOfSortedRangesIntersectingRange:(NSRange)testRng inIndexes:(NSRange)indexRange ;
- (NSRange) unionSortedRanges ;

@end




/***********************************
 * Mutable Range Array Primitives
 ***********************************/

@interface NSMutableArray (NisusRangeArrayPrimitives)

- (void) insertRange:(NSRange)rng atIndex:(NSUInteger)idx ;
- (void) removeRangeAtIndex:(NSUInteger)idx ;

@end




@interface NSMutableArray (NisusRangeArray)

- (void) addRange:(NSRange)rng ;
- (void) replaceRangeAtIndex:(NSUInteger)idx withRange:(NSRange)rng ;
- (BOOL) adjustRangesForEditInRange:(NSRange)editedRange delta:(NSInteger)delta ;

// these methods assume the range is sorted
- (void) insertSortedRange:(NSRange)rng ;

@end



