//
//  NSDate-NisusAdditions.h
//  WriterMerged
//
//  Created by Martin on 2008.07.25.
//  Copyright 2008 Nisus Software, Inc. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <NisusCore/NisusDefines.h>

// This data type is only used to represent time intervals since the system reference date, ie: [NSDate timeIntervalSinceReferenceDate]
// Generally this is used to calculate the elapsed time between two events.
typedef NSTimeInterval NisusTimeMark;
#define NisusTimeMark_NEVER (0)

static NISUS_INLINE NisusTimeMark NisusTimeMark_Now()
{
	return [NSDate timeIntervalSinceReferenceDate];
}

// This can return a negative value if the early mark is not actually an earlier time.
static NISUS_INLINE NSTimeInterval NisusTimeMark_SecondsBetweenMarks( NisusTimeMark earlyMark, NisusTimeMark currentMark )
{
	return (currentMark - earlyMark);
}

static NISUS_INLINE NSTimeInterval NisusTimeMark_SecondsSinceMark( NisusTimeMark earlyMark )
{
	return NisusTimeMark_SecondsBetweenMarks( earlyMark, NisusTimeMark_Now() );
}

static NISUS_INLINE NisusTimeMark NisusTimeMark_MarkByAddingSeconds( NisusTimeMark mark, NSTimeInterval addSeconds )
{
	return (mark + addSeconds);
}




@interface NSDateFormatter (NisusAdditions)

+ (NSDateFormatter*) dateFormatterWithFormat:(NSString*)fmt ;
+ (NSString*) stringFromDate:(NSDate*)date withFormat:(NSString*)fmt ;

@end



#define NTCalendarUnits_DATE (NSCalendarUnitYear | NSCalendarUnitMonth | NSCalendarUnitDay)
#define NTCalendarUnits_TIME (NSCalendarUnitHour | NSCalendarUnitMinute | NSCalendarUnitSecond)
#define NTCalendarUnits_DATE_TIME (NTCalendarUnits_DATE | NTCalendarUnits_TIME)

@interface NSDateComponents (NisusCore)
+ (instancetype) nowGregorianNisus ;
+ (instancetype) dateComponentsNisusWithYear:(NSInteger)year month:(NSInteger)month day:(NSInteger)day ;
+ (instancetype) dateComponentsNisusWithYear:(NSInteger)year month:(NSInteger)month day:(NSInteger)day hour:(NSInteger)hour minute:(NSInteger)min second:(NSInteger)sec timeZone:(NSTimeZone*)tz ;
@end



@interface NSDate (NisusAdditions)

+ (NSDate*) dateWithGregorianNisusComponents:(NSDateComponents*)comps ;
+ (NSDate*) dateWithGregorianNisusYear:(NSInteger)year month:(NSInteger)month day:(NSInteger)day ;
+ (NSDate*) dateWithGregorianNisusYear:(NSInteger)year month:(NSInteger)month day:(NSInteger)day hour:(NSInteger)hour minute:(NSInteger)min second:(NSInteger)sec timeZone:(NSTimeZone*)tz ;

+ (NSTimeInterval) secondsForTickCount:(UInt32)tickCount ;
+ (UInt32) tickCountForSeconds:(NSTimeInterval)secs ;

- (NSDateComponents*) componentsInGregorianCalendarNisus ;

- (NSString*) stringValueForPlistNisus ;
+ (NSDate*) dateWithStringValueFromPlistNisus:(NSString*)plistString ;

@end



@interface NSUserDefaults (NisusDateAdditions)

- (NSDate*) dateForKeyNisus:(NSString*)key ;
- (void) setDateNisus:(NSDate*)date forKey:(NSString*)key ;

@end


