//
//  NSFileManager-NisusAdditions.h
//  NisusToolbox
//
//  Created by Charles Jolley on Thu Jun 26 2003.
//  Copyright (c) 2003 Nisus Software, Inc.. All rights reserved.
//
@class NSColor;

typedef enum {
	NisusFileOpResult_Success				= 0,	// operation completed successfully
	NisusFileOpResult_FileDoesNotExist		= 1,	// operation failed because (source) file does not exist
	NisusFileOpResult_FileExists			= 2,	// operation failed because (destination) file already exists
	NisusFileOpResult_FolderDoesNotExist	= 3,	// operation failed because destination folder (or ancestor thereof) does not exist
	NisusFileOpResult_PermissionDenied		= 4,	// operation failed because not enough permissions
	NisusFileOpResult_Unknown				= 5,	// operation failed for an unknown reason
} NisusFileOpResult;

#define NisusUnicode_FilePathDelimiter ('/')



@interface NSFileManager (NisusAdditions)

- (NSURL*) userHomeFolderURL ;
- (NSURL*) userLibraryFolderURL ;
- (NSString*) pathByAbbreviatingRealHomeWithTilde:(NSString*)path ;
- (NSString*) pathByExpandingTildeToRealHome:(NSString*)path ;

- (NSURL*) applicationSupportProductFolderURL ;
- (NSURL*) applicationSupportSubfolderURLWithName:(NSString*)subfolderName create:(BOOL)isCreate ;

- (NSURL*) sandboxContainersFolderURL ;
- (NSURL*) applicationSandboxFolderURL ;
- (BOOL) isFileInsideSandboxAtURL:(NSURL*)testURL ;

- (BOOL) isItemInsideDocumentRevisionsFolderAtURL:(NSURL*)url ;
- (BOOL) isFolderEffectivelyEmptyNisusAtURL:(NSURL*)folderURL ;

// uniquing
- (NSString*) uniqueFileName:(NSString*)targetName inFolderAtURL:(NSURL*)folderURL ;
- (NSString*) uniqueFileName:(NSString*)targetName withSuffix:(NSString*)suffixFmt inFolder:(NSString*)folderPath ;
- (NSString*) uniqueFileName:(NSString*)targetName inFolder:(NSString*)folderPath ;
- (NSString*) uniqueFilePathForPath:(NSString*)targetPath ;
- (NSURL*) uniqueFileURLWithFileName:(NSString*)targetName inFolderAtURL:(NSURL*)folderURL ;

- (NSURL*) systemFolderURLOutsideSandboxNisusForDirectoryType:(NSSearchPathDirectory)dirType ;

- (BOOL) createDirectoryRecursiveAtPath:(NSString*)folderPath attributes:(NSDictionary*)attr ;
- (BOOL) createDirectoryAtPath:(NSString*)folderPath recursive:(BOOL)isRecursive attributes:(NSDictionary*)attr ;

- (void) touchFile:(NSString*)filePath ;
- (NSURL*) temporaryItemURLWithLastPathComponent:(NSString*)name error:(NSError**)errorPtr ;
- (NSString*) tempFilePathWithFileName:(NSString*)name ;
- (BOOL) accessTemporaryItemURLWithLastPathComponent:(NSString*)name forReplacingURL:(NSURL*)replaceURL error:(NSError**)clientError using:(BOOL(^)(NSURL* tempFileURL, NSError** visitorError))visitor ;

// display names
- (NSString*) displayNameAtURLNisus:(NSURL*)url ;
- (NSString*) nisusDisplayNameForPathComponent:(NSString*)comp ;
- (NSString*) nisusPathComponentForDisplayName:(NSString*)name convertIllegal:(BOOL)isConvert ;

// file operations
- (NisusFileOpResult) moveFileAtPath:(NSString*)srcPath toPath:(NSString*)destPath ;
- (NisusFileOpResult) copyFileAtPath:(NSString*)srcPath toPath:(NSString*)destPath ;
- (NisusFileOpResult) trashFileAtPath:(NSString*)path ;

- (BOOL) removeFileAtPath:(NSString*)path errorMessage:(NSString**)errMessPtr ;
- (NSDictionary*) nisusAttributesForFileAtPath:(NSString*)path traverseLink:(BOOL)isResolveLink ;
- (BOOL) visitFilesInFolderAtURL:(NSURL*)folderURL options:(NSDirectoryEnumerationOptions)ops getError:(NSError**)errorPtr using:(void(^)(NSURL* fileURL, BOOL* stop))visitor ;
- (BOOL) nisusSetAttributes:(NSDictionary*)attrs forFileAtPath:(NSString*)path errorMessage:(NSString**)errMessPtr ;

// permissions
- (BOOL) hasPosixPermissionToRead:(BOOL)isRead itemAtURL:(NSURL*)url ;

- (BOOL) isReadableItemAtURL:(NSURL*)url error:(NSError**)errorPtr ;
- (BOOL) isWritableFileAtURL:(NSURL*)url error:(NSError**)errorPtr ;
- (BOOL) isFileHiddenAtPath:(NSString*)path ;

// working with aliases/symlinks
- (NSString*) fullyResolvedPathForFileAtPath:(NSString*)path ;

- (BOOL) isSymlinkAtPath:(NSString*)path ;
- (BOOL) isAliasAtPath:(NSString*)path ;
- (NSString*) resolvedPathForAliasAtPath:(NSString*)aliasPath ;

- (BOOL) createSymlinkAtPath:(NSString*)destPath withDestinationPath:(NSString*)sourcePath ;

// extended file attributes
- (NSSet*) readExtendedAttributeNamesForFileAtURL:(NSURL*)fileURL error:(NSError**)errorPtr ;
- (BOOL) readExtendedAttribute:(NSString*)key getByteCount:(NSUInteger*)sizePtr getData:(NSData**)dataPtr forFileAtURL:(NSURL*)fileURL error:(NSError**)errorPtr ;
- (NSString*) readStringExtendedAttribute:(NSString*)xattrKey forFileAtURL:(NSURL*)fileURL error:(NSError**)errorPtr ;
- (BOOL) writeExtendedAttribute:(NSString*)key data:(NSData*)value forFileAtURL:(NSURL*)fileURL error:(NSError**)errorPtr ;

- (NSDate*) lastAccessedDateForFileAtPath:(NSString*)path ;
- (NSDate*) lastUsedDateNisusForFileAtURL:(NSURL*)url ;
- (NSColor*) labelColorForFileAtURL:(NSURL*)url ;
- (BOOL) isTrashedFileNisusAtURL:(NSURL*)testURL ;

// sizes
- (unsigned long long) sizeOfAllFilesAtPath:(NSString*)path ;
- (unsigned long long) sizeTotalNisusForItemAtURL:(NSURL*)fileURL ;

#if NISUS_DEBUG
+ (void) nisusWriteTempFileData:(NSData*)data ;
+ (void) nisusWriteTempFileData:(NSData*)data withFileName:(NSString*)name ;
#endif

@end
