//
//  NSFont+NisusCore.h
//  Toolbox
//
//  Created by Martin on Tue Nov 04 2003.
//  Copyright (c) 2003 Nisus Software, Inc.. All rights reserved.
//

#import <Foundation/Foundation.h>


// types and methods that help with managing constants Apple has not exposed
extern NSString* NisusFontFeatureKey_Type;
extern NSString* NisusFontFeatureKey_Value;

typedef enum {
	NisusFontFeatureType_ChangeCase = kLetterCaseType,
	NisusFontFeatureType_VerticalPosition = kVerticalPositionType,
	
	// This is used by Garamond Premier Pro to selectively make small caps of lowercase/uppercase letters, but only on 10.8 and above.
	NisusFontFeatureType_LowercaseToSmallCaps = 37, // value is 0 or 1
	NisusFontFeatureType_UppercaseToSmallCaps = 38, // value is 0 or 1
} NisusFontFeatureType;

typedef enum {
	NisusFontFeatureValue_ChangeCaseNone = kUpperAndLowerCaseSelector,
	NisusFontFeatureValue_ChangeCaseUppercase = kAllCapsSelector,
	NisusFontFeatureValue_ChangeCaseLowercase = kAllLowerCaseSelector,
	NisusFontFeatureValue_ChangeCaseSmallCaps = kSmallCapsSelector,
	NisusFontFeatureValue_ChangeCaseCapitalize = kInitialCapsSelector,
	NisusFontFeatureValue_ChangeCaseSmallCapsAlt = 12, // used by Garamond Premier, Arno, Bodoni, and more (on 10.7). Should not be used by NTCharacterCaseAttributeName, only used by Apple code or external sources
	NisusFontFeatureValue_INVALID = 999,
} NisusFontFeatureValue_ChangeCase;
extern BOOL NisusFontFeatureValue_ChangeCaseIsValid( NisusFontFeatureValue_ChangeCase value );
extern NisusFontFeatureValue_ChangeCase NisusFontFeatureValue_ChangeCaseForFeatureValue( NisusFontFeatureType featureType, NSUInteger value );
extern NisusFontFeatureValue_ChangeCase NisusFontFeaturesChangeCaseValueForFeatures( NSArray* features, NSArray** cleanedFeatsPtr );

typedef enum {
	NisusFontFeatureValue_VerticalPositionNormal = kNormalPositionSelector,
	NisusFontFeatureValue_VerticalPositionSuper = kSuperiorsSelector,
	NisusFontFeatureValue_VerticalPositionSub = kInferiorsSelector,
	NisusFontFeatureValue_VerticalPositionOrdinal = kOrdinalsSelector,
	NisusFontFeatureValue_VerticalPositionFraction = 4, // undocumented?
} NisusFontFeatureValue_VerticalPosition;

extern NSArray* NisusFontFeaturesByAdding( NSArray* features, NisusFontFeatureType type, NSUInteger value, BOOL isAdditive );
extern NSArray* NisusFontFeaturesByRemoving( NSArray* features, NisusFontFeatureType type );
extern BOOL NisusFontFeatures_GetValueForFeature( NSArray* features, NisusFontFeatureType type, NSUInteger* valuePtr );




@interface NSFont (NisusAdditions)

+ (NSFont*) fontWithFamilyName:(NSString*)family ;

// Getting specific fonts. These never return nil.
+ (NSFont*) lucidaGrandeFontOfSize:(CGFloat)size ;
+ (NSFont*) boldLucidaGrandeFontOfSize:(CGFloat)size ;
+ (NSFont*) timesFontOfSize:(CGFloat)size ;
+ (NSFont*) helveticaFontOfSize:(CGFloat)size ;
+ (NSFont*) monospaceFontOfSize:(CGFloat)size ;
+ (NSFont*) menuItemFont ;

// descriptor attribute access
- (NSArray*) fontFeatures ;
- (NSFont*) fontByAddingFontFeautures:(NSArray*)features ;
- (NSFont*) fontByAddingFontFeature:(NisusFontFeatureType)featType value:(NSUInteger)value ;

- (NSFont*) fontByAddingChangeCaseFontFeature:(NisusFontFeatureValue_ChangeCase)useCase ;

// serialization
- (id) initWithPropertyListRepresentation:(id)momento ;
- (id) propertyListRepresentation ;

// generating glyphs
- (NSData*) glyphsForCharactersInString:(NSString*)str withRange:(NSRange)charRng ;
- (BOOL) isHyphenationGlyph:(NSGlyph)checkGlyph cache:(NSMutableDictionary*)cache ;

@end


@interface NSFontManager (NisusAdditions)
- (NSFont*) fontWithFamilyName:(NSString*)family ;
- (NSFont*) fontCoveringCharacter:(unichar)ch ofSize:(CGFloat)size ;
- (NSFont*) fontCoveringCharacter:(unichar)ch ofSize:(CGFloat)size preferredFontName:(NSString*)prefName ;
- (NSArray*) fontsCoveringCharacter:(unichar)ch ofSize:(CGFloat)size ;
@end

