/* 
================================================================================
File:    NSObject+NisusCore.h
Project: NisusToolboxAuthor:  cjolley
Date:    Wed Jun 19 2002 
================================================================================
Copyright (c) 2002, Nisus Software, Inc.

*/

#import <Cocoa/Cocoa.h>



@interface NSObject (NisusAdditions)

+ (instancetype) conditionalCastNisus:(id)obj ;
- (BOOL) isClassObject ;

#if NISUS_DEBUG
- (NSString*) debugState ;
- (NSString*) debugStateIvars ;
#endif

- (NSNotificationCenter*) notificationCenter ;

// method replacement
+ (NSSet*) classSelectorNames ;
+ (NSSet*) instanceSelectorNames ;
+ (Class) classImplementingSelectorNisus:(SEL)findSel ;

+ (IMP) swizzleSelector:(SEL)sel toUseImplementation:(IMP)newImp ;
+ (IMP) swizzleSelector:(SEL)sel toUseImplementationOfSelector:(SEL)newImpSel ;
+ (IMP) swizzleClassSelector:(SEL)sel toUseImplementation:(IMP)newImp ;

+ (BOOL) addImplementation:(IMP)newImp withSelector:(SEL)selector types:(const char*)types ;
+ (BOOL) addImplementation:(IMP)newImp withSelectorName:(const char*)selName types:(const char*)types ;
+ (BOOL) addImplementation:(IMP)newImp withSelectorName:(const char*)selName zeroArgumentsAndReturnType:(const char*)retType ;
+ (BOOL) addImplementation:(IMP)newImp withSelectorName:(const char*)selName oneArgumentWithType:(const char*)argType returnType:(const char*)retType ;

+ (IMP) swizzleDirectlyOrAddSelector:(SEL)replaceSel toUseImplementation:(IMP)useImp ;
+ (IMP) swizzleDirectlyOrAddSelector:(SEL)replaceSel toUseImplementationOfSelector:(SEL)useSel ;

// ivars
- (id) valueForIvarWithNameNisus:(const char*)ivarName directInClass:(Class)cls ;

// associated objects
- (id) associatedObjectForKey:(void*)key ;
- (void) setAssociatedObject:(id)value forKey:(void*)key ;
- (void) setAssociatedObjectWeakly:(id)value forKey:(void*)key ;

// conditional sending
- (id) performSelectorIfResponds:(SEL)sel ;
- (id) performSelectorIfResponds:(SEL)sel withObject:(id)obj ;
- (id) performSelectorIfResponds:(SEL)sel withObject:(id)obj1 withObject:(id)obj2 ;

// Variants on performSelector that handle more primitive types
- (CGFloat) floatForPerformSelector:(SEL)sel ;
- (CGFloat) floatForPerformSelector:(SEL)sel withObject:(id)obj ;

- (NSInteger) intForPerformSelector:(SEL)sel ;
- (NSInteger) intForPerformSelector:(SEL)sel withObject:(id)obj ;

- (BOOL) boolForPerformSelector:(SEL)sel ;
- (BOOL) boolForPerformSelector:(SEL)sel withObject:(id)obj ;

- (id) performSelector:(SEL)sel withFloat:(CGFloat)value ;
- (id) performSelector:(SEL)sel withInt:(NSInteger)value ;
- (id) performSelector:(SEL)sel withBool:(BOOL)value ;

@end


/*** an object that does nothing but provide a unique pointer value. Only compares as equal to itself (same pointer) ***/
@interface NTUniqueObject : NSObject
@end

