//
//  NSObject+NisusCoreKVO.h
//  InfoClick
//
//  Created by Martin Wierschin on 2012.09.21.
//  Copyright (c) 2012 Nisus. All rights reserved.
//
// This helper object makes registering for KVO notifications easier.
// The implementation is more expensive (requiring additional objects for each registration), but it's easier to use in terms of dispatch/cleanup code.

#import <Foundation/Foundation.h>


@interface NSObject (NisusCoreKVO)

// These methods register an observer just like Apple's method, but with some key differences:
//
// 1. When your observer deallocs, all registrations for it that were added through this API will automatically be removed. 
//	  THIS IS NOT TRUE of the object being observed! If that object deallocs, you MUST call removeObserver before dealloc finishes.
//
// 2. This API fixes some possible problems with convoluting super/self by using a unique observer/context.
//
// 3. Dispatch is done through a convenient specific selector automatically. When using a selector it should take the form:
//	  - (void) observedObjectChanged:(id)changedObject change:(NSDictionary*)changeInfo ;
//
- (void) addObserverNisus:(NSObject*)anObserver forKeyPath:(NSString*)keyPath options:(NSKeyValueObservingOptions)options notifySelector:(SEL)sel ;
- (void) removeObserverNisus:(NSObject*)anObserver forKeyPath:(NSString*)keyPath ;

@end

