/* 
================================================================================
File:    NSString-NisusAdditions.h
Project: NisusToolboxAuthor:  cjolley
Date:    Fri Jun 14 2002 
================================================================================
Copyright (c) 2002, Nisus Software, Inc.

*/

#import <Cocoa/Cocoa.h>
#import <NisusCore/NisusMutationState.h>


@interface NSMutableString (ExposedCocoaPrivate)
- (void) replaceCharactersInRange: (NSRange) r withCString: (char*) s length: (NSUInteger) len;
@end


@interface NSString (NisusCore)

+ (NSString*) uuidString ;

+ (NSString*) stringWithCharacter:(unichar)ch ;
+ (NSString*) stringWithData:(NSData*)data encoding:(NSStringEncoding)enc ;
+ (NSString*) stringWithBytes:(const void*)bytes length:(NSUInteger)byteCount encoding:(NSStringEncoding)enc ;

+ (NSString*) stringWithNisusInt:(NSInteger)value ;
+ (NSString*) stringWithNisusFloat:(CGFloat)value ;
+ (NSString*) stringWithNisusFloat:(CGFloat)value maxDecimalCount:(NSUInteger)decimalCount ;

+ (NSString*) stringWithFourCharCode:(FourCharCode)code ;
- (FourCharCode) fourCharCodeValue ;

- (BOOL) getPascalString:(Str255)pstringPtr ;
- (BOOL) getPascalString:(StringPtr)pstringPtr ofLength:(NSUInteger)pstrLen ;

// file paths
- (BOOL) hasPathExtensionNisus:(NSString*)testExt ;
- (NSString*) stringByChangingPathExtensionNisus:(NSString*)newExt ;

- (NSString*) stringByPathEquivalence ;
- (BOOL) isPathEquivalent:(NSString*)otherPath ;
- (BOOL) isPathInFolder:(NSString*)folderPath ;
- (BOOL) isPathDescendantOf:(NSString*)folderPath ;

#if NISUS_DEBUG
// debug
- (NSString*) descriptionForNisusLogWithMaxLength:(NSUInteger)maxLen ;
#endif

// some convenience methods
- (NSComparisonResult) nisusUserDisplayCompare:(NSString*)other ;

- (NSString*) stringByMatchingCase:(NSString*)match ;
- (NSString*) stringByPrettyTruncationToLength:(NSUInteger)maxLen ;
- (NSString*) stringByPrettyTruncationToLength:(NSUInteger)maxLen allowLineBreak:(BOOL)isAllowBreaks truncationSuffix:(NSString*)suffix ;
- (NSString*) stringByHardWrappingToLineLength:(NSUInteger)lineLen ;
- (NSString*) toggledcaseString;

- (NSRange) rangeOfWordAtIndex:(NSUInteger)charIdx ;
- (NSRange) rangeOfCaseChangeWordAtIndex:(NSUInteger)charIdx ;
- (NSRange) rangeOfTrimmedWordForRange:(NSRange)range ;
- (NSRange) paragraphContentRangeForRange:(NSRange)rng ;

- (NSRange) rangeOfComposedCharactersForRange:(NSRange)rng ;
- (NSUInteger) locationOfCharacterFromSet:(NSCharacterSet*)set options:(NSUInteger)ops location:(NSUInteger)start ;

- (NSString*) stringByReplacingAllOccurrencesOfString:(NSString*)findStr withString:(NSString*)newStr ;
- (NSString*) stringByReplacingAllOccurrencesOfCharactersInSet:(NSCharacterSet*)findSet withString:(NSString*)newStr ;
- (NSString*) stringByReplacingAllLineEndingsWithString:(NSString*)newString ;
- (NSString*) stringByLossyConversionToEncoding:(NSStringEncoding)enc ;

- (NSArray*) componentsSeparatedByCharactersInSet:(NSCharacterSet*)cset withEscapeCharacter:(unichar)escapeCh ;

- (unsigned) hexIntValue ;
- (unsigned) hexIntValueAtIndex:(NSUInteger)start effectiveRange:(NSRange*)effRngPtr ;
- (int) intValueAtIndex:(NSUInteger)start effectiveRange:(NSRange*)effRngPtr ;
- (float) floatValueAtIndex:(NSUInteger)start effectiveRange:(NSRange*)effRngPtr ;
- (int) parseInt:(int*)intPtr orDouble:(double*)floatPtr fromRange:(NSRange*)rngPtr currency:(BOOL*)currencyPtr ;

// more conviencence
- (unichar*) characterBufferNisus ;
- (NSSet*) setWithComponentsSeparatedByString:(NSString*)sep ;

- (BOOL) containsStringNisus:(NSString*)needle ;
- (BOOL) containsStringNisus:(NSString*)needle options:(NSStringCompareOptions)ops ;

- (NSString *)stringByRemovingSurroundingWhitespace ;

- (NSRange) nonwhitespaceRangeForRange:(NSRange)rng ;

@end

@interface NSArray (NisusStringAdditions)
- (NSString*) componentsJoinedByCharacter:(unichar)joinCh withEscapeCharacter:(unichar)escapeCh ;
@end



@interface NSMutableString (NisusAdditions)

typedef NSRange (^NisusStringReplacementMatcher)(NSString* searchString, NSRange searchRange, NSString** replacementStrPtr);
+ (NSUInteger) replaceAllOptimized:(NisusMutationState*)statePtr usingMatcher:(NisusStringReplacementMatcher)matcher ;

- (NSUInteger) replaceAllOccurrencesOfCharactersInSet:(NSCharacterSet *)set withString:(NSString*) replaceString ;
- (BOOL) replaceAllOccurrencesOfString:(NSString*)oldString withString:(NSString *)newString;
- (void) removeCharactersInSet:(NSCharacterSet*)set ;

- (void) deleteSurroundingWhitespace ;

// The following methods were originally borrowed from OmniFoundation, copright OmniGroup
- (void) appendCharacter:(unichar)chr;

@end


@interface NSString (NisusDrawingAdditions)
- (NSRect) nisusBoundingRectWithAttributes:(NSDictionary*)attrs ;
- (void) nisusDrawAtPoint:(NSPoint)drawPt withAttributes:(NSDictionary*)attrs ;
- (void) nisusDrawInFrame:(NSRect)frame withAttributes:(NSDictionary*)attrs ;
- (NSRect) drawNisusCenteredInRect:(NSRect)rect withAttributes:(NSDictionary*)attrs ;
@end




