//
//  NisusAssertionManager.h
//  NisusCore
//
//  Created by Martin on 2013.08.29.
//  Copyright (c) 2013 Nisus. All rights reserved.
//
// This class is thread-safe.

#import <Foundation/Foundation.h>
#import <NisusCore/NisusAssertions.h>
#import <NisusCore/NSDate+NisusCore.h>
@class NisusAssertionInfo;

#if NISUS_CORE_ASSERTIONS_ENABLED

typedef void (^NisusAssertionReporterBlock)( NisusAssertionInfo* failedAssertInfo );




@interface NisusAssertionManager : NSObject {
	NisusAssertionReporterBlock _reporterBlock;
	
	NisusAssertionInfo* _currentAssert; // assert currently being processed.
	NisusAssertionInfo* _lastAssert;
	NSUInteger _floodCount;
	NisusTimeMark _disableUntilTimeMark;
	NSMutableDictionary* _priorAssertMap; // key is equality key, value is NisusAssertionInfo 
	
	NSUInteger _disableReporterLevel;
	NSMutableArray* _collected;
	NSMutableArray* _opTitles;
}

+ (instancetype) instance ;


// assertion reporting
@property (copy) NisusAssertionReporterBlock reporterBlock ;

- (BOOL) isErrorReportingEnabled ;

- (void) beginDisableErrorReporting ;
- (void) endDisableErrorReporting ;

- (void) pushOperationTitle:(NSString*)title ;
- (void) popOperationTitle ;


// collect failed asserts instead of triggering them
- (BOOL) isCollectingAssertions ;
- (void) collectFailedAssertion:(NisusAssertionInfo*)assert ;
- (NSArray*) collectAssertionsWhileDoing:(void(^)())block ;

@end

#endif // NISUS_CORE_ASSERTIONS_ENABLED
