//
//  NisusDate.h
//  NisusCore
//
//  Created by Martin on 2016.10.25.
//  Copyright © 2016 Nisus. All rights reserved.
//
// This class represents a date using Gregorian date components.
// Each instance is guaranteed to have a defined (year, month, day) and can always be resolved to an NSDate.
//
// Optionally a time can be stored, again using components (hour, minutes, seconds).
// By default the time zone is set to the default / local time zone.
//
// This class was introduced partly as a replacement for the deprecated NSCalendarDate,
// but also because it's annoying to always convert between NSDate and NSDateComponents.

#import <Foundation/Foundation.h>



@interface NisusDate : NSObject <NSCopying> {
	NSDate* _date;
	NSDateComponents* _dateComps;
}

+ (instancetype) now ;

+ (instancetype) dateWithDate:(NSDate*)date ;
+ (instancetype) dateWithDateComponents:(NSDateComponents*)comps ;
+ (instancetype) dateWithYear:(NSInteger)year month:(NSUInteger)month day:(NSUInteger)day ;
+ (instancetype) dateWithYear:(NSInteger)year month:(NSUInteger)month day:(NSUInteger)day hour:(NSUInteger)hour minute:(NSUInteger)minute second:(NSUInteger)second timeZone:(NSTimeZone*)zone ;

- (instancetype) initWithDate:(NSDate*)date ;
- (instancetype) initWithDateComponents:(NSDateComponents*)comps ;

- (NSComparisonResult) compare:(NisusDate*)other ;

- (NSDate*) date ;
- (NSDateComponents*) dateComponents ;
- (NSTimeZone*) timeZone ;

@end
