//
//  NisusExceptionHandler.h
//  Toolbox
//
//  Created by Martin Wierschin on Fri Jul 16 2004.
//  Copyright (c) 2004 Nisus Software, Inc. All rights reserved.

#import <Foundation/Foundation.h>

#if NISUS_DEV_BUILD
	// disable report, as not not so helpful while in debugger
	#define NisusExceptionHandler_ENABLED 0
#else
	#if NISUS_DEBUG
		#define NisusExceptionHandler_ENABLED 1 
	#else
		// don't include in release builds, as cocoa appkit exceptions are wonky
		#define NisusExceptionHandler_ENABLED 0
	#endif
#endif



@interface NisusExceptionHandler : NSObject {
	BOOL _enabled;
}

+ (instancetype) instance ;
+ (void) enableReportAllUncaughtExceptions ;

- (void) reportException:(NSException*)excep ;

- (void) beginDisableExceptionReporting ;
- (void) endDisableExceptionReporting ;

+ (void) disableExceptionReportingWhileTryingBlock:(void(^)())runCode catchUsing:(void(^)(NSException* excep))catchBlock finally:(void(^)())finallyBlock ;
+ (void) disableExceptionReportingWhileTryingBlock:(void(^)())runCode catchUsing:(void(^)(NSException* excep))catchBlock ;

@end



@interface NSException (NisusExceptionHandler)
- (void) logWithTitle:(NSString*)logTitle ;
@end



// The following macros are functionally equivalent to using "disableExceptionReportingWhileTryingBlock:etc:"
// These are more efficient because they will be no-ops in release builds.
// However, no matter what kind of build, this will always add a try-catch block that stops exceptions from bubbling up.
//
// These work by wrapping regular code blocks and can be used as stand-ins for the old NS_DURING etc macros.
//
#if NisusExceptionHandler_ENABLED
	#define NisusExceptionHandler_IGNORE_EXCEPTIONS_TRY \
		[NisusExceptionHandler disableExceptionReportingWhileTryingBlock:^{
			
	#define NisusExceptionHandler_IGNORE_EXCEPTIONS_END_NO_CATCH \
		} catchUsing:NULL];

	#define NisusExceptionHandler_IGNORE_EXCEPTIONS_CATCH \
		} catchUsing:^(NSException* localException){

	#define NisusExceptionHandler_IGNORE_EXCEPTIONS_END_CATCH \
		}];
#else
	// Release build versions. Should just try-catch without disabling exception handler.
	#define NisusExceptionHandler_IGNORE_EXCEPTIONS_TRY \
		@try {
			
	#define NisusExceptionHandler_IGNORE_EXCEPTIONS_END_NO_CATCH \
		} @catch( NSException* localException ) {}

	#define NisusExceptionHandler_IGNORE_EXCEPTIONS_CATCH \
		} @catch( NSException* localException ) {

	#define NisusExceptionHandler_IGNORE_EXCEPTIONS_END_CATCH \
		}
#endif



// These macros are just like the above, but do NOT add a @try block
#if NisusExceptionHandler_ENABLED
	#define NisusExceptionHandler_DISABLE_REPORTING_BEGIN \
		[NisusExceptionHandler disableExceptionReportingWhileTryingBlock:^{
			
	#define NisusExceptionHandler_DISABLE_REPORTING_END \
		} catchUsing:NULL];
#else
	#define NisusExceptionHandler_DISABLE_REPORTING_BEGIN
	#define NisusExceptionHandler_DISABLE_REPORTING_END
#endif


