//
//  NisusFloatArray.h
//  Toolbox
//
//  Created by Martin Wierschin on Wed Apr 07 2004.
//  Copyright (c) 2004 Nisus Software, Inc. All rights reserved.

// Object wrapper for an array of floats. 
// Array is fixed in capacity at creation, but float values inside are mutable.


#import <Foundation/Foundation.h>


@interface NisusFloatArray : NSObject <NSCopying> {
    CGFloat* _floats;
    NSUInteger _capacity;
    NSUInteger _count; // Number of floats. For us always equals capacity, but subclasses may override.
}

- (id) initWithCapacity:(NSUInteger)count ;
- (id) initWithFloats:(const CGFloat*)dataPtr count:(NSUInteger)count ; // desig init

- (NSUInteger) count ;
- (const CGFloat*) floatBuffer ;

// accessors
- (CGFloat) floatAtIndex:(NSUInteger)idx ;
- (void) getFloats:(CGFloat*)buffer inRange:(NSRange)floatRange ;
- (void) setFloat:(float)value atIndex:(NSUInteger)idx ;
- (void) addFloat:(float)delta toFloatAtIndex:(NSUInteger)idx ;

- (void) replaceFloatAtIndex:(NSUInteger)idx withFloat:(CGFloat)f ;
- (void) replaceFloatsInRange:(NSRange)rng withFloatsFrom:(const CGFloat*)fPtr ;

// calcs
- (CGFloat) sumFloats ;
- (CGFloat) sumFloatsInRange:(NSRange)rng ;

// NSArray convienence... expects/returns NSNumbers that have a float value.
- (id) objectAtIndex:(NSUInteger)idx ;
- (void) replaceObjectAtIndex:(NSUInteger)idx withObject:(id)obj ;

@end



/*** Stores floats but can grow/shrink in size as needed. ***/
@interface NisusFloatVector : NisusFloatArray {
}

- (void) addFloat:(CGFloat)f ;
- (void) insertFloat:(CGFloat)f atIndex:(NSUInteger)idx ;

- (void) removeFloatAtIndex:(NSUInteger)idx ;
- (void) removeAllFloats ;

@end
