//
//  NisusGeometryUtil.h
//  NisusCore
//
//  Created by Martin Wierschin on 2013.10.04.
//  Copyright (c) 2013 Nisus. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "NisusUtil.h"


/***************************************/
/**** working with points and sizes ****/
/***************************************/

// point diff
static NISUS_INLINE NSPoint NisusPoint_SubtractPoint( NSPoint pt, NSPoint minusPt )
{
	NSPoint result = NSMakePoint( pt.x - minusPt.x, pt.y - minusPt.y );
	return result;
}

// CARTESIAN distance
CGFloat NisusDistanceBetweenPoints( NSPoint pt1, NSPoint pt2 );

// adjusting points
static NISUS_INLINE NSPoint NisusShiftPoint( NSPoint pt, CGFloat deltaX, CGFloat deltaY ) {
	pt.x += deltaX;
	pt.y += deltaY;
	return pt;
}

static NISUS_INLINE NSPoint NisusShiftPointByPoint( NSPoint pt, NSPoint delta ) {
	pt.x += delta.x;
	pt.y += delta.y;
	return pt;
}

// ensures size is not bigger than limit
NSSize NisusSize_ConstrainProportionally( NSSize aSize, NSSize limitSize );




/*****************************/
/*** coordinates of a rect ***/
/*****************************/
static NISUS_INLINE NSPoint NisusRect_MiddlePoint( NSRect rect ) {
	return NSMakePoint(NSMidX(rect), NSMidY(rect));
}

static NISUS_INLINE NSPoint NisusRect_TopLeftPoint( NSRect rect ) {
	return NSMakePoint(NSMinX(rect), NSMinY(rect));
}
static NISUS_INLINE NSPoint NisusRect_TopRightPoint( NSRect rect ) {
	return NSMakePoint(NSMaxX(rect), NSMinY(rect));
}
static NISUS_INLINE NSPoint NisusRect_BottomLeftPoint( NSRect rect ) {
	return NSMakePoint(NSMinX(rect), NSMaxY(rect));
}
static NISUS_INLINE NSPoint NisusRect_BottomRightPoint( NSRect rect ) {
	return NSMakePoint(NSMaxX(rect), NSMaxY(rect));
}

static NISUS_INLINE NSPoint NisusRect_MiddleTopPoint( NSRect rect ) {
	return NSMakePoint(NSMidX(rect), NSMinY(rect));
}
static NISUS_INLINE NSPoint NisusRect_MiddleBottomPoint( NSRect rect ) {
	return NSMakePoint(NSMidX(rect), NSMaxY(rect));
}
static NISUS_INLINE NSPoint NisusRect_MiddleLeftPoint( NSRect rect ) {
	return NSMakePoint(NSMinX(rect), NSMidY(rect));
}
static NISUS_INLINE NSPoint NisusRect_MiddleRightPoint( NSRect rect ) {
	return NSMakePoint(NSMaxX(rect), NSMidY(rect));
}


/*****************************/
/****** rect operations ******/
/*****************************/

extern NSRect NisusRect_OutsetXY( NSRect rect, CGFloat dx, CGFloat dy );
extern NSRect NisusRect_OutsetEdges( NSRect origRect, CGFloat outsetLeft, CGFloat outsetRight, CGFloat outsetTop, CGFloat outsetBottom );

// sentinel rect value
static NISUS_INLINE NSRect NisusRect_UnsetRect() {
	return NSMakeRect(MAXFLOAT,MAXFLOAT,0,0);
}

static NISUS_INLINE BOOL NisusRect_IsUnset( NSRect rect ) {
	return (MAXFLOAT == rect.origin.x) && (MAXFLOAT == rect.origin.y);
}

BOOL NisusRect_IsRectsIntersect( NSRect rect1, NSRect rect2 );
NSRect NisusRect_RectEnclosingPoints( NSPoint pt1, NSPoint pt2 );
NSRect NisusRect_RectEnclosingRectAndPoint( NSRect rect, NSPoint pt );
NSRect NisusRect_UnionRects( NSRect rect1, NSRect rect2 );

NSRect NisusRect_ClipStartOfRect( NSRect rect, CGFloat dx, CGFloat dy );
NSRect NisusRect_ClipEndOfRect( NSRect rect, CGFloat dx, CGFloat dy );
BOOL NisusRect_SubtractRects( NSRect wholeRect, NSRect subtractRect, NSRect* topPtr, NSRect* rightPtr, NSRect* bottomPtr, NSRect* leftPtr );

static NISUS_INLINE NSRect NisusRect_ApplyFactor( NSRect sourceRect, CGFloat factor )
{
	NSRect destRect = sourceRect;
	destRect.origin.x *= factor;
	destRect.origin.y *= factor;
	destRect.size.width *= factor;
	destRect.size.height *= factor;
	return destRect;
}

#if NISUS_DEBUG
NSString* NisusRect_StringForSize( NSSize size );
NSString* NisusRect_StringForRect( NSRect rect );
NSString* NisusRect_StringForRectDiff( NSRect mainRect, NSRect logRect );
NSString* NisusRect_StringForFloatCoordinate( CGFloat coord );
#endif



/*****************************/
/***** drawing / display *****/
/*****************************/

// aligns coordinates so they fall on integral pixel boundaries (prevent fuzziness)
#define NisusIntegralDisplayCoordinate( coord ) (floor(coord) + 0.5)
#define NisusIntegralDisplayPoint( pt ) NSMakePoint( floor((pt).x) + 0.5, floor((pt).y) + 0.5 )
NSRect NisusIntegralDisplayRect( NSRect rect );
CGFloat NisusIntegralDisplayCoordinateForThickness( CGFloat coord, CGFloat th );

// checks for equality assuming that values represent visual coordinates, and thus allow for a small fudge factor
#define NisusEqualDisplayTolerance (0.1)
#define NisusIsEqualDisplayCoordinates( f1, f2 ) (NisusIsEqualFloats((f1), (f2), NisusEqualDisplayTolerance))

BOOL NisusIsEqualDisplayPoints( NSPoint pt1, NSPoint pt2 ) ;
BOOL NisusIsEqualDisplaySizes( NSSize sz1, NSSize sz2 ) ;
BOOL NisusIsEqualDisplayRects( NSRect rect1, NSRect rect2 ) ;



