//
//  NTWeakIvars.h
//  WriterMerged
//
//  Created by Martin Wierschin on 2013.06.28.
//  Copyright (c) 2013 __MyCompanyName__. All rights reserved.
//

#import <objc/runtime.h>

#if NISUS_DEBUG
	#define NISUS_ENABLE_WEAK_IVAR_DEBUG 1
#else
	#define NISUS_ENABLE_WEAK_IVAR_DEBUG 0
#endif





/*** These macros help with declaring/accessing weak (unretained) ivars ***/
// Instead of accessing an ivar value directly, always use a corresponding macro instead.
// The only gotcha has to do with writing -dealloc methods; you must call NISUS_WEAK_IVAR_CLEAR_IN_DEALLOC in your dealloc methods.
#define NISUS_WEAK_IVAR_ATTRIBUTES assign

#if NISUS_ENABLE_WEAK_IVAR_DEBUG
	Class NisusGetClassDirectly( id object );
	BOOL NisusIsPointerAnObjCClass( void* pointer );

	extern void NisusDisableWeakObjectWarnings();

	// do not call directly
	extern id _NisusWeakIvar_GetValue( id holder, id ivar, Class expectedClass );
	extern void _NisusWeakIvar_SetValue( id holder, id* ivarStorage, Class* expectedClassStorage, id newValue );
	extern void _NisusWeakIvar_Clear( id holder, id* ivarStorage, Class* expectedClassStorage );
	extern NSString* _NisusWeakIvar_GetDebugDescription( id holder, id ivar, Class expectedClass );

	// managing weak ivar values
	#define NISUS_WEAK_IVAR_DECLARE(ivarName)			ivarName;				Class ivarName ## _origClass
	#define NISUS_WEAK_IVAR_GET(ivarName)				_NisusWeakIvar_GetValue( self, self->ivarName, (ivarName ## _origClass) )
	#define NISUS_WEAK_IVAR_SET(ivarName, expr)			_NisusWeakIvar_SetValue( self, &(self->ivarName), &(ivarName ## _origClass), (expr) )
	#define NISUS_WEAK_IVAR_CLEAR(ivarName)				_NisusWeakIvar_Clear( self, &(self->ivarName), &(ivarName ## _origClass) )
	#define NISUS_WEAK_IVAR_CLEAR_IN_DEALLOC(ivarName)	NISUS_WEAK_IVAR_CLEAR(ivarName)
	#define NISUS_WEAK_IVAR_CLEAR_COPIED(obj,ivarName)	obj->ivarName = nil;	obj->ivarName ## _origClass = nil
	#define NISUS_WEAK_IVAR_DEBUG_DESC(ivarName)		_NisusWeakIvar_GetDebugDescription( self, self->ivarName, (ivarName ## _origClass) )

#else
	// deployment build should just use pointer-value assignment
	#define NISUS_WEAK_IVAR_DECLARE(ivarName)			ivarName
	#define NISUS_WEAK_IVAR_GET(ivarName)				self->ivarName
	#define NISUS_WEAK_IVAR_SET(ivarName, expr)			self->ivarName = (expr)
	#define NISUS_WEAK_IVAR_CLEAR(ivarName)				self->ivarName = nil
	#define NISUS_WEAK_IVAR_CLEAR_IN_DEALLOC(ivarName)	
	#define NISUS_WEAK_IVAR_CLEAR_COPIED(obj,ivarName)	obj->ivarName = nil
#endif

// Use in .m to avoid retyping these macros.
#define NISUS_WEAK_IVAR_SYNTHESIZE(ivarType, ivarName, getter, setter)	\
	- (ivarType) getter {												\
		return NISUS_WEAK_IVAR_GET(ivarName);							\
	}																	\
																		\
	- (void) setter(ivarType)newValue {									\
		NISUS_WEAK_IVAR_SET(ivarName, newValue);						\
	}

// these operations do not require we actually talk to the weak ivar, so are always safe
#define NISUS_WEAK_IVAR_IsEqualPointer(ivarName, otherPointer) \
	(ivarName == (otherPointer))

#define NISUS_WEAK_IVAR_IsNil(ivarName) \
	NISUS_WEAK_IVAR_IsEqualPointer(ivarName, nil)


