//
//  NisusLeaks.h
//  WriterMerged
//
//  Created by Martin Wierschin on 2013.05.02.
//  Copyright (c) 2013 __MyCompanyName__. All rights reserved.
//
// Some things to help when profiling for leaks / heapshot.

#import <Foundation/Foundation.h>

#define NisusLeaks_IS_PROFILING (defined(NISUS_LEAK_CHECK))
#define NisusLeaks_USING_GUARD_MALLOC (0)

#if NISUS_DEV_BUILD
	#define NisusLeaks_DEBUG_MAJOR_LEAKS (1)
#else
	#define NisusLeaks_DEBUG_MAJOR_LEAKS (0)
#endif


// This flag can be checked if some service should be disabled because we're running leak detection or some other process that is too slow for high-flux allocations.
#define NisusLeaks_DISABLE_HIGH_FLUX_ALLOCATIONS (NisusLeaks_IS_PROFILING || NisusLeaks_USING_GUARD_MALLOC)

// Whether we should prefer to call release/invalidation ASAP, instead of delaying it for major objects.
#define NisusLeaks_RELEASE_ASAP (NisusLeaks_IS_PROFILING || NisusLeaks_USING_GUARD_MALLOC)



#if NisusLeaks_DEBUG_MAJOR_LEAKS
@interface NisusLeaks : NSObject

+ (void) ensureObjectDeallocates:(id)obj afterDelay:(NSTimeInterval)delay failureMessage:(NSString*)mess ;

+ (void) incrementInstanceCountForHighFluxObject:(id)object ;
+ (void) decrementInstanceCountForHighFluxObject:(id)object ;

@end

	#define NisusLeaks_ENSURE_DEALLOCS(object, delaySeconds) [NisusLeaks ensureObjectDeallocates:(object) afterDelay:(delaySeconds) failureMessage:nil]
	#define NisusLeaks_INCREMENT_INSTANCE_COUNT(object) [NisusLeaks incrementInstanceCountForHighFluxObject:(object)]
	#define NisusLeaks_DECREMENT_INSTANCE_COUNT(object) [NisusLeaks decrementInstanceCountForHighFluxObject:(object)]

#else
	#define NisusLeaks_ENSURE_DEALLOCS(object, delaySeconds)
	#define NisusLeaks_INCREMENT_INSTANCE_COUNT(object)
	#define NisusLeaks_DECREMENT_INSTANCE_COUNT(object)
#endif

