//
//  NisusSystemVersions.h
//  WriterMerged
//
//  Created by Martin on 2011.08.05.
//  Copyright 2011 Nisus. All rights reserved.
//

#import <Cocoa/Cocoa.h>

// Info on system version, eg: OSX 10.9 (Mavericks) would be major=10, minor=9, rev=0
typedef struct {
	NSUInteger major;
	NSUInteger minor;
	NSUInteger rev;
} NisusSystemVersion;

void NisusSystemVersion_Get( NisusSystemVersion* versionPtr );
NSComparisonResult NisusSystemVersion_Compare( NisusSystemVersion* aaaPtr, NisusSystemVersion* bbbPtr );

NSString* NisusSystemVersion_ToString( NisusSystemVersion* versionPtr );
NSString* NisusSystemVersion_PrettyString(); // pretty system version string
BOOL NisusSystemVersion_GetFromString( NisusSystemVersion* versionPtr, NSString* parseString );



// misc system info
NSString* NisusSystem_AppKitVersionString();
NSString* NisusSystem_PrimaryLanguageCode(); // eg: "en"
NSString* NisusSystem_ComputerName();
NSString* NisusSystem_RootVolumeName();
NSString* NisusSystem_CurrentUserName();

// hardware info
NSString* NisusSystem_MacModelName();
unsigned long long NisusSystem_RAM(); // byte count

BOOL NisusSystem_CPUIs64Bit();
NSUInteger NisusSystem_CPUCount();
NSUInteger NisusSystem_CPUSpeed(); // MHz



// macro is true when on that platform or later (eg: panther check also true if tiger)
// 10.12 macOS Sierra
// 10.11 El Capitan
// 10.10 Yosemite
// 10.9	Mavericks
// 10.8 Mountain Lion
// 10.7 Lion
// 10.6 Snow Leopard
BOOL NisusSystem_IsSystemVersionAtLeast( NSUInteger major, NSUInteger minor, NSUInteger rev );
BOOL NisusSystem_IsSystemVersionLessThan( NSUInteger major, NSUInteger minor, NSUInteger rev );

#define NTAppKitVersionNumber10_9 1265

#define NisusSystem_IS_OSX_10_10		(floor(NSAppKitVersionNumber) > NTAppKitVersionNumber10_9)
#define NisusSystem_IS_OSX_10_9			(floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_8)
#define NisusSystem_IS_OSX_10_8			(NSAppKitVersionNumber >= NSAppKitVersionNumber10_8)
#define NisusSystem_IS_OSX_10_7			(NSAppKitVersionNumber >= NSAppKitVersionNumber10_7)




/*** Architecture stuff ***/


@interface NSApplication (NisusSystemAdditions)
- (NSString*) runningArchitectureString ;
@end
