//
//  NisusTriState.h
//  WriterMerged
//
//  Created by Martin on 2011.03.04.
//  Copyright 2011 Nisus. All rights reserved.
//
// Type with three possible states: YES, NO, and unset.
// Default/zero state is "unset".

#import <Foundation/Foundation.h>


typedef enum {
	NisusTriState_Unset	= 0,
	NisusTriState_NO	= -1,
	NisusTriState_YES	= 1,
} NisusTriState;


#define NisusTriStateIsSet(state)		(NisusTriState_Unset != (state))
#define NisusTriStateIsUnset(state)		(NisusTriState_Unset == (state))

#define NisusTriStateToBOOL(value)		(((value) == NisusTriState_YES) ? YES : NO)
#define NisusTriStateFromBOOL(value)	((value) ? NisusTriState_YES : NisusTriState_NO)

// If the first tri value is set then returns it, otherwise returns the second tri value.
// This is a short-circuit operator in that "second" will only be eval'd if the first tri value is unset.
#define NisusTriStateFirst(first, second)	\
	return (NisusTriStateIsSet(first) ? (first) : (second))

// Return YES if and only if the state is set and exactly matches the given BOOL.
static inline BOOL NisusTriStateEqualsBool( NisusTriState state, BOOL test )
{
	if( NisusTriState_Unset == state ) {
		return NO;
	}
	else {
		return (NisusTriStateToBOOL(state) == test);
	}
}
