//
//  NisusUnicodeUtil.h
//  NisusCore
//
//  Created by Martin on 2013.10.11.
//  Copyright (c) 2013 Nisus. All rights reserved.
//

#import <Foundation/Foundation.h>


// represents a single Unicode code point
typedef UInt32 NisusUnicodePoint;




typedef enum {
	NisusUnicodeSurrogateType_None	= 0,	// UTF16 value is not a surrogate (eg: represents code points U+0000 to U+FFFF).
	NisusUnicodeSurrogateType_Low	= 1,	// UTF16 value is a surrogate holding the low bits of the code point
	NisusUnicodeSurrogateType_High	= 2,	// holds high bits of code point
} NisusUnicodeSurrogateType;

// Maximum expansion that can occur when conversion is performed (per original codepoint character).
#define NisusCodepointToUnicharExpansionFactor (2)

// Returns YES if the given codepoint requires a surrogate pair when represented in UTF16.
static NISUS_INLINE BOOL NisusCodepointRequiresUnicharSurrogatePair( NisusUnicodePoint code )
{
	return (code >= 0x10000);
}

// Determines if the given UTF16 value is part of a surrogate pair.
static NISUS_INLINE NisusUnicodeSurrogateType NisusSurrogateTypeForUnichar( unichar ch )
{
    if( 0xD800 != (ch & 0xF800) ) return NisusUnicodeSurrogateType_None;
	return (0 == (ch & 0x0400)) ? NisusUnicodeSurrogateType_High : NisusUnicodeSurrogateType_Low;
}

// Combines the high and low UTF16 surrogate pair into a code point.
static NISUS_INLINE NisusUnicodePoint NisusConvertUnicharsToCodepoint( unichar high, unichar low )
{
	NisusUnicodePoint value = (NisusUnicodePoint)(high & 0x3FF) << 10;
	value = value | (low & 0x3FF);
    return 0x10000 + value;
}

// Converts a full unicode codepoint into UTF16 characters. Returns the number of unichars placed in the given buffer.
// The buffer must be able to accomodate at least NisusCodepointToUnicharExpansionFactor characters.
static NISUS_INLINE NSUInteger NisusConvertCodepointToUnichars( NisusUnicodePoint code, unichar* outPtr )
{
	if( code <= 0xFFFF ) {
		outPtr[0] = (unichar)code;
		return 1;
    } 
	else {
		// convert to UTF16 surrogate pair
		NisusUnicodePoint suppCode = code - 0x10000;
		outPtr[0] = 0xD800 | (suppCode & 0xFFC00) >> 10;	// high surrogate
		outPtr[1] = 0xDC00 | (suppCode & 0x003FF);			// low surrogate
		return 2;
    }
}

