//
//  WordNetManager.h
//

#import <Foundation/Foundation.h>

typedef enum {
	words = 0,
	antonyms, hypernyms, hyponyms, holonyms, meronyms, attributes,
	entailments, causes, seealso, verbgroup, 
	similarities, participles, pertainyms,
	nominalizations,
	Relation_Class, Relation_Classification,
	frames, 
	gloss,
	RELATION_COUNT,
} Relation;

typedef enum {
	noun = 1,
	verb,
	adj,
	adv
} POS;
#define POS_MIN 1
#define POS_MAX 4


@interface WordNetManager : NSObject {
    NSBundle* _dataBundle;
    NSMutableDictionary *_indexDict, *_dataDict;
    NSArray *_nounRoots, *_verbRoots;
	NSArray* _allRoots;
	
    FILE *indexFiles[POS_MAX + 1];
    FILE* _dataFiles[POS_MAX + 1];
    FILE *excFiles[POS_MAX + 1];
}

+ (WordNetManager*) defaultManager ;

- (id)initWithBundle:(NSBundle *)bundle;
- (NSBundle *)dataBundle;

- (NSArray *)wordsForSynset:(NSNumber *)synset;
- (NSArray *)synsetsForWord:(NSString *)word;
- (NSArray *)synsetsForInflectedWord:(NSString *)word;
- (NSNumber*) randomSynset ;

- (POS)posForSynset:(NSNumber *)synset;
- (NSString *)avatarForSynset:(NSNumber *)synset;
- (NSString *)avatarForSynset:(NSNumber *)synset withConstraint:(NSString *)word;
- (NSString *)synsetPrefix:(NSNumber *)synset toAvatar:(NSString *)avatar;
- (NSString *)posPrefixForSynset:(NSNumber *)synset;
- (NSString *)posPrefixForSynset:(NSNumber *)synset toAvatar:(NSString *)avatar;
- (NSString *)dropPrefixForAvatar:(NSString *)avatar;

- (NSString *)glossForSynset:(NSNumber *)synset;

- (NSArray *)allHypernymRoots;
- (NSArray *)hypernymNounRoots;
- (NSArray *)hypernymVerbRoots;

- (NSNumber *)hypernymForSynset:(NSNumber *)synset;
- (NSArray *)hyponymsForSynset:(NSNumber *)synset;
- (NSArray *)multipleHypernymsForSynset:(NSNumber *)synset;
- (NSArray *)ancestryForSynset:(NSNumber *)synset;
- (NSNumber*) parentForSynset:(NSNumber*)synset ;

- (NSArray *)similaritiesToSynset:(NSNumber *)synset;
- (NSArray *)antonymsForSynset:(NSNumber *)synset;

- (NSArray *)framesForSynset:(NSNumber *)synset;

- (NSArray *)meronymsForSynset:(NSNumber *)synset;
- (NSArray *)holonymsForSynset:(NSNumber *)synset;

- (NSArray *)causesForSynset:(NSNumber *)synset;
- (NSArray *)entailmentsForSynset:(NSNumber *)synset;

- (NSArray *)seeAlsoForSynset:(NSNumber *)synset;
- (NSArray *)verbGroupForSynset:(NSNumber *)synset;
- (NSArray *)attributesForSynset:(NSNumber *)synset;

- (NSArray *)participlesForSynset:(NSNumber *)synset;
- (NSArray *)pertainymsForSynset:(NSNumber *)synset;

@end
